@@ -40,6 +40,7 @@
 #include "gok-modifier-keymasks.h"
 #include "gok-gconf-keys.h"
 #include "gok-word-complete.h"
+#include "gok-keyboard.h"
 
 /*
  * NOTE: we can tweak the efficiency/accuracy of the
@@ -255,6 +256,8 @@ void gok_spy_accessible_unref(Accessible* accessible)
 	gok_log_leave ();
 }
 
+extern SPIBoolean cspi_accessible_is_a (Accessible *accessible, const char *interface_name);
+
 gboolean gok_spy_accessible_is_okay(Accessible* accessible)
 {
 	gboolean returncode = FALSE;
@@ -1048,7 +1051,9 @@ gboolean gok_spy_check_queues(void)
 			break;
 		case GOKSPY_STATE_EVENT:
 			gok_log("GOKSPY_STATE_EVENT");
-			((AccessibleChangeListenerCB)(m_ptheStateChangeListener))(en->event->source);
+			if (m_ptheStateChangeListener) {
+				((AccessibleChangeListenerCB)(m_ptheStateChangeListener))(en->event->source);
+			}
 			break;
 		case GOKSPY_WINDOW_ACTIVATE_EVENT:
 			gok_log("GOKSPY_WINDOW_ACTIVATE_EVENT");
@@ -1058,7 +1063,9 @@ gboolean gok_spy_check_queues(void)
 				gok_spy_accessible_unref (m_ptheWindowAccessible);
 				m_ptheWindowAccessible = en->event->source;
 				gok_spy_accessible_ref (m_ptheWindowAccessible);
-				((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
+				if (m_ptheWindowChangeListener) {
+					((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
+				}
 			}
 			break;
 		case GOKSPY_DEFUNCT_EVENT:
@@ -1069,7 +1076,9 @@ gboolean gok_spy_check_queues(void)
 			gok_spy_accessible_unref (m_ptheAccessibleWithText);
 			m_ptheWindowAccessible = NULL;
 			m_ptheAccessibleWithText = NULL;
-			((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
+			if (m_ptheWindowChangeListener) {
+				((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
+			}
 			break;
 		case GOKSPY_CONTAINER_EVENT:
 			gok_log("GOKSPY_CONTAINER_EVENT");
@@ -2262,7 +2271,7 @@ gok_spy_get_toolbar_item (Accessible *toolbar_child)
 }
 
 static GokSpyUIFlags 
-gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboard_ui_flags)
+gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboard_ui_flags, AccessibleNodeFlags context_flags)
 {
 	Accessible* child;
 	Accessible* parent;
@@ -2282,16 +2291,24 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 		gok_log ("not worth searching.");
 		gok_log_leave ();
 		return ui_flags;
-	}	
+	}			
 	parent_role = Accessible_getRole (parent);
 	child_count = Accessible_getChildCount (parent);
+	
+	/* assign context */
+	if (parent_role == SPI_ROLE_HTML_CONTAINER) {
+			context_flags.data.inside_html_container = TRUE;
+	}
+	
 	/* MAX_CHILD_COUNT is a bit of a kludge and will cause certain table items to be unreachable via UI Grab */
 	if (child_count > MAX_CHILD_COUNT)
 		child_count = MAX_CHILD_COUNT;
 	for (i = 0; i < child_count; ++i)
 	{
-		AccessibleNodeFlags flags;
+		AccessibleNodeFlags flags;  /* TODO? memcpy context_flags? */
 		flags.value = 0;
+		flags.data.inside_html_container = context_flags.data.inside_html_container;
+
 		child = Accessible_getChildAtIndex (parent, i);
 		if (child)
 		{
@@ -2322,7 +2339,7 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 				}
 				else
 				{
-				    ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags).value;
+				    ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags, context_flags).value;
 				}
 			}
 			if ((keyboard_ui_flags.data.gui || keyboard_ui_flags.data.editable_text) && 
@@ -2345,12 +2362,12 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 				    /* special case for notebook tabs, whose children should also appear */
   				    if (role == SPI_ROLE_PAGE_TAB)
   				    {
-  					ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags).value;
+  					ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags, context_flags).value;
   				    }
 				}
 				else
 				{
-					ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags).value;
+					ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags, context_flags).value;
 				}
 			}
 			gok_spy_accessible_unref (child);
@@ -2384,12 +2401,14 @@ GokSpyUIFlags
 gok_spy_update_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboard_ui_flags)
 {
 	GokSpyUIFlags ui_flags;
+	AccessibleNodeFlags context_flags;
 
 	gok_log_enter ();
 
 	gok_spy_free_nodes (_priv_ui_nodes);
 	_priv_ui_nodes = NULL;
-	ui_flags = gok_spy_search_component_list (rootAccessible, keyboard_ui_flags);
+	
+	ui_flags = gok_spy_search_component_list (rootAccessible, keyboard_ui_flags, context_flags);
 	/* _priv_ui_nodes is now re-assigned, by above call. */
 	gok_spy_resolve_namesakes (_priv_ui_nodes);
 
@@ -2471,4 +2490,3 @@ gboolean gok_spy_accessible_is_desktopChild(Accessible* accessible)
 	gok_log_leave();
 }
 */
-
